/**
 * Copyright (c) Members of the EGEE Collaboration. 2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 * Authors: Oscar Koeroo, Mischa Sall\'e, Aram Verstegen
 *      NIKHEF Amsterdam, the Netherlands
 *      <grid-mw-security@nikhef.nl>	
 */

#define _XOPEN_SOURCE	600

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <limits.h>

#include "environ.h"

/**
 * returns number of entries in NULL terminated string array, such as argv and
 * environ(7). In case list is NULL it also returns 0.
 */
size_t cgul_strarrlen(const_env_t list)  {
    size_t entries;

    /* Check if list exists: prevents common segfault situation */
    if (!list) return 0;
    /* Now loop over its entries */
    for(entries=0; list[entries]; entries++);

    return entries;
}

/**
 * returns the number of entries in list which start with pattern. If pattern of
 * list is NULL it also returns 0.
 */
size_t cgul_patternentries(const_env_t list, const_string_t pattern) {
    size_t i,patlen,match=0;

    /* Check if list exists: prevents common segfault situation */
    if (!list || !pattern)
	return 0;
    patlen=strlen(pattern);
    /* Now loop over its entries */
    for(i=0; list[i]; i++)	{
	if (strncmp(list[i],pattern,patlen)==0)
	    match++;
    }
    return match;
}

/**
 * Returns the length of the variable name part of the var=value pair.
 * It returns 0 when either namevalue==NULL, when it contains no '=', or when
 * it starts with '='.
 */
size_t cgul_getvarnamelen(const_string_t namevalue)    {
    char *pos;

    /* find first '=' */
    if (!namevalue) /* empty input */
	return 0;
    pos=strchr(namevalue,'=');
    if (pos==NULL) /* not var=value */
	return 0;
    /* when input is '=value' we return 0, which is ok */
    return (strlen(namevalue)-strlen(pos)); /* no pointer arithmetics */
}


/**
 * Finds name for given var=value pair in namevalue. varname is a buffer
 * which will contain the result and has space for buffersize chars.
 * Returns 0 on error, or length of env varname, *also* when length >=
 * buffersize. In that case nothing is copied as it would not fit in the buffer
 * (length is excl null byte).
 */
size_t cgul_getvarname(char *varname, const size_t buffersize,
		    const_string_t namevalue)  {
    size_t varnamelen=cgul_getvarnamelen(namevalue);

    if (varnamelen==0) /* Empty name */
	return 0;

    /* Only copy when it fits, note: we need one extra for the \0 byte */
    if (varnamelen<buffersize)   {
	strncpy(varname,namevalue,varnamelen);
	varname[varnamelen]='\0';
    }

    /* return length, even when nothing is copied */
    return varnamelen;
}

/**
 * Finds index in **src matching variable name, in case of no match (including
 * env==NULL) returns -1, when match>INT_MAX returns -3.
 */
int cgul_getvarindex(const_env_t src, const_string_t varname)   {
    size_t i,len;

    if (src==NULL || varname==NULL)
	return -1;

    len=strlen(varname);
    /* Loop over env */
    for (i=0; src[i]; i++)  {
	if (strncmp(src[i],varname,len)==0 && src[i][len]=='=')
	    /* Found match */
	    return (i>INT_MAX ? -3 : (int)i);
    }
    /* NO match found */
    return -1;
}

/**
 * Finds the value of varname in src list. Returns NULL when no match, incl.
 * NULL valued src or varname.
 */
char *cgul_getenv_src(env_t src, char *varname)	{
    size_t i,len;
    char *match;

    /* Don't use getvarindex, since this saves us one strlen (-: */
    if (src==NULL || varname==NULL)
	return NULL;

    len=strlen(varname);
    /* Loop over env */
    for (i=0; src[i]; i++)  {
	if (strncmp(src[i],varname,len)==0 && src[i][len]=='=')	{
	    /* Found match */
	    match=&(src[i][len+1]);
	    return match;
	}
    }
    /* NO match found */
    return NULL;
}

/**
 * As unsetenv(3) but using var=value pair, specified in namevalue. The maximum
 * var length is MAXVARNAMELEN. See also putenv(3). 
 * Returns integer, 0 on success, negative on error: -2 on length of varname too
 * long, -1 on invalid input.
 */
int cgul_unsetenvpair(const_string_t namevalue) {
    char varname[MAXENVVARNAMELEN];
    size_t rc=cgul_getvarname(varname,MAXENVVARNAMELEN,namevalue);

    /* check result */
    if ( rc==0 )    /* invalid input */
	return -1;
    if ( rc>=MAXENVVARNAMELEN)	/* too large */
	return -2;

    /* Unset variable */
    if (unsetenv(varname))
	return -1;
    
    return 0;
}

/**
 * Removes a name=value pair from dst as in cgul_unsetenvpair() with a 'external
 * environment'. Return 0 on success, -1 on error.
 */
int cgul_unsetenvpair_dst(env_t *dst, const_string_t namevalue)  {
    size_t i,entry=0; /* initialize entry to silence the compiler */
    env_t newenv=*dst;
    int found=0;

    /* Check arguments */
    if (namevalue==NULL)
	return -1;

    /* look for existing entry */
    for (i=0; newenv[i]; i++)  {
	if (strcmp(namevalue,newenv[i])==0)   {
	    found=1;
	    entry=i;
	    break;
	}
    }
    /* no match */
    if (found==0) return 0;
    
    /* Remainder of list... */
    for (; newenv[i]; i++);

    /* Now clean entry if it's there and swap with the last in the list */
    free(newenv[entry]);

    /* If there were more than one elements, move the last to the removed one */
    if (i>0)    {
	newenv[entry]=newenv[i-1]; /* last element */
	newenv[i-1]=NULL;
    } else
	newenv[entry]=NULL;
    return 0;
}

/**
 * Clears the entire environment
 * Returns integer, negative on error (-2 name too long, -1 other error), 0 on
 * success
 */
int cgul_clear_env(void)	{
    int rc;
    while(environ[0])	{
	/* cgul_unsetenvpair() returns -2 when the name is too long, we then
	 * will NOT unsetenv the value */
	if ( (rc=cgul_unsetenvpair(environ[0])) )
	    return rc;
    }
    return 0;
}

/*
 * Clears the environment of all variables starting with pattern, case
 * INsensitive.
 * Returns number of cleaned entries, -1 on error, -2 on name too long, INT_MAX
 * on more values than INT_MAX (env is still cleaned in that case).
 */
int cgul_clear_env_pattern(const_string_t pattern)	{
    int rc;
    size_t i=0, match=0, patlen;

    if (!pattern)
	return -1;
    patlen=strlen(pattern);
    while(environ[i])	{
	if (strncasecmp(pattern,environ[i],patlen)==0)  {
	    if ( (rc=cgul_unsetenvpair(environ[i])) )
		/* Note: cgul_unsetenvpair returns -2 when the name is too long,
		 * we then cannot unsetenv the pair */
		return rc;
	    match++;
	} else
	    i++; /* only increase when we haven't just removed an entry... */
    }
    /* Check that match fits in a INT_MAX */
    return (match>INT_MAX ? INT_MAX : (int)match);
}


/**
 * Creates a duplicate of src. In case *dst!=NULL it will be used and should be
 * big enough. Otherwise space is malloc-ed. dst is only updated upon success.
 * Returns -1 upon error, 0 on success.
 */
int cgul_copy_env(env_t *dst, const_env_t src)	{
    size_t i,j,envlen;
    env_t newenv=*dst;

    /* If no dst exists, malloc it. */
    if (newenv==NULL)  {
	envlen=cgul_strarrlen(src);
	newenv=(env_t)malloc((envlen+1)*sizeof(char*));
	if (newenv==NULL)
	    return -1;
    }

    /* Copy src to newenv */
    for (i=0; src[i]; i++)  {
	if ( (newenv[i]=strdup(src[i]))==NULL )  {
	    /* Error: cleanup memory and reset */
	    for (j=0; j<i; j++)
		free(newenv[j]);
	    if (*dst==NULL)
		free(newenv);
	    return -1;
	}
    }
    /* Finalize the new environ */
    newenv[i]=NULL;
    if (*dst==NULL)
	*dst=newenv;

    return 0;
}

/**
 * Same as copy_env() but first unsets all environment variables starting with
 * MALLOC, case INsensitive. dst is again only updated upon success.
 * Returns -1 upon general error, -2 on varname too long, 0 on success.
 */
int cgul_safe_copy_env(env_t *dst, const_env_t src) {
    int rc;

    /* Clean all MALLOC* variables */
    rc=cgul_clear_env_pattern("MALLOC");
    if (rc<0)
	return rc;

    return cgul_copy_env(dst,src);
}

/**
 * Creates a list of the var=value pairs from src which have varname in list.
 * If it fails NULL is returned. This can also be done using add_src_list()
 * using dst==NULL, but this is more efficiently coded. All new entries are
 * strdup-ed from those in src.
 */
env_t cgul_copy_src_list(const_env_t src, const_env_t list)    {
    int src_idx;
    size_t i,j, match=0, listlen=cgul_strarrlen(list);
    env_t newenv;

    /* Malloc space */
    newenv=(env_t)malloc((listlen+1)*sizeof(char*));
    if (newenv==NULL)
	return NULL;
    
    /* IMPORTANT: list must be NULL terminated! */
    newenv[0]=NULL;

    /* Only do when list exists, otherwise return empty list, but not NO list!
     * Note it's not an error, just an empty list. */
    if (src==NULL || list==NULL)
	return newenv;

    /* loop over list: Note we can have unmatched entries: match counts */
    for (i=0; list[i]; i++)   {
	/* Look if list[i] is in src */
	if ( (src_idx=cgul_getvarindex((const_env_t )src,list[i]))
		>= 0)  { 
	    /* Don't! check for duplicates... */
	    /* substitute existing: first try strdup */
	    if ((newenv[match]=strdup(src[src_idx]))==NULL)   {
		/* error: cleanup */
		for (j=0; j<match; j++) {
		    free(newenv[j]); newenv[j]=NULL;
		}
		free(newenv);
		return NULL;
	    }
	    match++;
	    /* Don't forget the final NULL pointer! */
	    newenv[match]=NULL;
	} else if (src_idx==-3)	{ /* matching entry is too large: tidy up */
	    /* cleanup */
	    for (j=0; j<match; j++) {
		free(newenv[j]); newenv[j]=NULL;
	    }
	    free(newenv);
	    return NULL;
	}
    }
    return newenv;
}

/**
 * Adds to dst the var=value pairs from src which have varname in list.
 * All entries will be strdup-ed from those in src.
 * If *dst is NULL, it will be malloc-ed, otherwise it will be realloc-ed.
 * Upon success returns 0. On error -1 is returned. If part of the environment
 * has been successfully copied, they will stay part of the environment.
 * See also copy_src_list() which is more efficient when dst is NULL !
 */
int cgul_add_src_list(env_t *dst, const_env_t src, const_env_t list) {
    int i,src_idx,dst_idx;
    env_t newenv=*dst;
    size_t dstlen,listlen,match;
    char *dummy;

    listlen=cgul_strarrlen(list);
    /* Create space for combined array, might be too much if list is not fully
     * used or partially replaces old entries, but more efficient (single
     * realloc() call).
     * Note that we check for empty list AFTER the malloc, and BEFORE the
     * realloc */
    if (newenv==NULL)  {
	dstlen=0;
	newenv=(env_t)malloc((listlen+1)*sizeof(char*));
	if (newenv==NULL)
	    return -1;
	*dst=newenv; /* update *dst after malloc */
	/* IMPORTANT: list must be NULL terminated! */
	newenv[0]=NULL;
	/* Only continue when list and src exist, do alloc in this case, since
	 * dst shouldn't return empty. Not an error. */
	if (list==NULL || src==NULL)
	    return 0;
    } else {
	/* Only do when list and src exist, no need to realloc, not an error. */
	if (list==NULL || src==NULL)
	    return 0;
	/* Count length of existing list */
	dstlen=cgul_strarrlen((const_env_t )*dst);
	newenv=(env_t)realloc(newenv,(dstlen+listlen+1)*sizeof(char*));
	if (newenv==NULL)
	    return -1;
	*dst=newenv; /* update *dst after realloc */
    }

    match=0;
    /* loop over list: Note we can have unmatched entries: match counts */
    for (i=0; list[i]; i++)   {
	/* Look if list[i] is in src */
	if ( (src_idx=cgul_getvarindex(src,list[i])) >= 0)	{
	    /* Make already a copy, we need it in any case */
	    if ( (dummy=strdup(src[src_idx])) == NULL)
		/* Out of mem: cannot cleanup anymore, might have
		 * substituted entries */
		return -1;
	    /* Look for list[i] in newenv: we might have it already. Note
	     * that this is not checked when using copy_src_list() */
	    if ( (dst_idx=
		    cgul_getvarindex((const_env_t)newenv,list[i]))
		    >= 0) {
		/* REPLACE: substitute existing: first try strdup */
		/* Now substitute: free old entry in newenv and link to new
		 * entry */
		free(newenv[dst_idx]);
		newenv[dst_idx]=dummy;
	    } else if (dst_idx==-1) {
		/* No match found: create new one */
		newenv[dstlen+match++]=dummy;/* created new entry: update match */
		newenv[dstlen+match]=NULL;   /* keep env NULL terminated */
	    } else  {
		/* index too large (-3), or unknown error */
		return -1;
	    }
	} else if (src_idx==-3)	{
	    /* src index too large */
	    return -1;
	}

    }
    return 0;
}

/**
 * Adds to dst the var=value pairs from src which start with pattern.
 * All entries will be strdup-ed from those in src.
 * If *dst is NULL, it will be malloc-ed, otherwise it will be realloc-ed.
 * Upon success returns the number of matches. On error -1 is returned, if part
 * of the environment has been successfully copied, these entries will stay part
 * of the environment.
 * An empty pattern (either "" or NULL) or src is NOT considered error.
 * See also cgul_add_src_list()
 */
int cgul_add_src_pattern(env_t *dst, const_env_t src, const_string_t pattern) {
    int not_matched;
    env_t newenv=*dst;
    size_t i,j,dstlen,patlen,numpat,match=0,namelen;
    char *namevaluecopy;

    /* numpat is 0 in case of no match, in that case newenv will simply be a
     * single NULL entry. */
    numpat=cgul_patternentries(src,pattern);
    /* Create space for combined array, might be too much if list is not fully
     * used or partially replaces old entries, but more efficient (single
     * realloc() call).
     * Note that we check for empty list AFTER the malloc, and BEFORE the
     * realloc */
    if (newenv==NULL)  {
	dstlen=0;
	newenv=(env_t)malloc((numpat+1)*sizeof(char*));
	if (newenv==NULL)
	    return -1;
	*dst=newenv; /* update *dst after malloc */
	/* IMPORTANT: list must be NULL terminated! */
	newenv[0]=NULL;
	/* Only continue when list and src exist, do alloc in this case, since
	 * dst shouldn't return empty. Not an error. */
	if (pattern==NULL || src==NULL)
	    return 0;
    } else {
	/* Only do when list and src exist, no need to realloc, not an error. */
	if (pattern==NULL || src==NULL)
	    return 0;
	/* Count length of existing list */
	dstlen=cgul_strarrlen((const_env_t )*dst);
	newenv=(env_t)realloc(newenv,(dstlen+numpat+1)*sizeof(char*));
	if (newenv==NULL)
	    return -1;
	*dst=newenv; /* update *dst after realloc */
    }
    /* Make sure all unset entries are NULL */
    for (i=dstlen+1; i<dstlen+numpat+1; i++) newenv[i]=NULL;

    patlen=strlen(pattern);
    /* loop over src */
    for (i=0; src[i]; i++)   {
	/* Look if src[i] matches pattern */
	if (strncmp(src[i],pattern,patlen)==0)	{
	    /* Look for length of name part (0 = means no name) */
	    if ( (namelen=cgul_getvarnamelen(src[i])) == 0)
		break;
	    /* Make already a copy, we need it in any case */
	    if ( (namevaluecopy=strdup(src[i])) == NULL)
		/* Out of mem: cannot cleanup anymore, might have substituted
		 * entries */
		return -1;
	    /* look for possibly existing entry */
	    not_matched=1;
	    for (j=0; not_matched && newenv[j]; j++)  {
		if (strncmp(newenv[j],namevaluecopy,namelen)==0 &&
		    newenv[j][namelen]=='=') {
		    /* REPLACE: substitute existing */
		    free(newenv[j]); /* free old */
		    newenv[j]=namevaluecopy; /* set new */
		    not_matched=0; /* found match -> done */
		}
	    }
	    if (not_matched)	{
		/* ADD: no old match */
		newenv[dstlen+match++]=namevaluecopy;/* created new entry:
							update match */
		newenv[dstlen+match]=NULL;	/* keep env NULL terminated */
	    }
	}
    }
    return (int)match;
}

/**
 * adds var=value pair to dst. If the variable already exists, it's entry is
 * replaced. Otherwise *dst is realloc-ed and a new entry is added, the
 * resulting array is returned.
 * Upon success returns 0. In case of error -1 is returned and dst remains
 * unchanged. namevalue is strdup-ed into dst and hence can be freed by the
 * caller (unlike putenv(3p) )
 */
int cgul_add_namevalue(env_t *dst, const_string_t namevalue)	{
    size_t i,dstlen,namelen;
    char *namevaluecopy;
    env_t newenv=*dst;

    /* Look for length of name part (no = means no name) */
    if ( (namelen=cgul_getvarnamelen(namevalue)) ==0)
	return -1;
    /* Make already a copy, we need it in any case, unless realloc fails */
    if ( (namevaluecopy=strdup(namevalue)) == NULL)
	return -1;
    /* look for existing entry */
    for (i=0; newenv[i]; i++)  {
	if (strncmp(namevalue,newenv[i],namelen)==0 &&
	    newenv[i][namelen]=='=')
	{
	    free(newenv[i]);
	    newenv[i]=namevaluecopy;
	    return 0;
	}
    }
    /* No old match, make a new entry, add two, because strarrlen doesn't count
     * the NULL string at the end. First create new entry, since that's easier
     * to undo. */
    dstlen=cgul_strarrlen((const char* const *)newenv);
    newenv=(env_t)realloc(newenv,(dstlen+2)*sizeof(char*));
    if (newenv==NULL)	{
	free(namevaluecopy);
	return -1;
    }
    *dst=newenv; /* update *dst after (re/m)alloc */
    newenv[dstlen]=namevaluecopy;
    newenv[dstlen+1]=NULL;

    return 0;
}

/**
 * creates a name=value pair from fmt and the remaining arguments as a printf
 * statement, then adds the name=value pair to the environment dst
 * Returns 0 on success, -1 on error.
 */
int cgul_env_printf_dst(env_t *dst, const char *fmt, ...) {
    int rc;
    va_list ap;

    /* get va_list */
    va_start(ap, fmt);

    /* Call cgul_env_vprintf_dst() for rest */
    rc=cgul_env_vprintf_dst(dst,fmt,ap);
    va_end(ap);

    return rc;
}

/**
 * creates a name=value pair from fmt and the remaining arguments as a vprintf
 * statement, then adds the name=value pair to the environment dst
 * Returns 0 on success, -1 on error.
 */
int cgul_env_vprintf_dst(env_t *dst, const char *fmt, va_list ap) {
    char *namevalue=NULL;
    int len,rc;
    va_list ap_copy;

    /* Create copy to forward to vsnprintf() */
    va_copy(ap_copy,ap);
    /* Determine length of namevalue pair */
    len=vsnprintf(namevalue,(size_t)0,fmt,ap_copy)+1; /* add 1 for \0 */

    /* Clear state and reinitialize */
    va_end(ap_copy);
    va_copy(ap_copy,ap);

    /* Check vsnprintf succeeded, can fail based on fmt */
    if (len<0)
	return -1;

    /* malloc mem and fill the string, len is guaranteed positive */
    if ((namevalue=(char*)malloc((size_t)len))==NULL)
	return -1;
    if (vsnprintf(namevalue,(size_t)len,fmt,ap_copy)<0)  {
	/* print: note if fmt doesn't match # in ap it's also an error */
	rc=-1;
    } else {
	/* add to the environment, rc is 0 on success, -1 on error */
	rc=cgul_add_namevalue(dst,namevalue);
    }
    free(namevalue);
    /* Call va_end just for safety */
    va_end(ap_copy);

    return rc;
}

/**
 * adds name=value pair to dst, as in setenv() with a 'external environment' .
 * If the variable already exists, it's entry is replaced. Otherwise *dst is
 * realloc-ed and a new entry is added.
 * Upon success returns 0. In case of error -1 is returned and dst remains
 * unchanged. name and value can be freed by the caller (unlike putenv(3p) )
 */
int cgul_setenv_dst(env_t *dst, const_string_t name, const_string_t value)  {
    size_t i,namelen,valuelen,totallen,dstlen;
    char *namevalue;
    const_string_t realvalue;
    env_t newenv=*dst;

    /* Check arguments */
    if (name==NULL)
	return -1;
    if (value==NULL)	{
	realvalue="";
	valuelen=0;
    } else  {
	realvalue=value;
	valuelen=strlen(realvalue);
    }
    namelen=strlen(name);
    /* Add two: '=' and '\0' */
    totallen=namelen+valuelen+2*sizeof(char);
    if ((namevalue=(char*)malloc(totallen))==NULL)
	return -1;
    sprintf(namevalue,"%s=%s",name,realvalue);

    /* look for existing entry */
    for (i=0; newenv[i]; i++)  {
	if (strncmp(name,newenv[i],namelen)==0 && newenv[i][namelen]=='=')   {
	    free(newenv[i]);
	    newenv[i]=namevalue;
	    return 0;
	}
    }
    /* No old match, make a new entry, add two, because strarrlen doesn't count
     * the NULL string at the end. */
    dstlen=cgul_strarrlen((const_env_t)newenv);
    newenv=(env_t)realloc(newenv,(dstlen+2)*sizeof(char*));
    if (newenv==NULL)	{
	free(namevalue);
	return -1;
    }
    *dst=newenv; /* update *dst after (re/m)alloc */
    newenv[dstlen]=namevalue;
    newenv[dstlen+1]=NULL;

    return 0;
}

/**
 * Removes a variable from dst as in unsetenv() with a 'external environment'.
 * Upon success returns 0. In case of error -1 is returned and dst remains
 * unchanged.
 */
int cgul_unsetenv_dst(env_t *dst, const_string_t name)  {
    size_t i,entry=0,namelen; /* initialize entry to silence compiler */
    env_t newenv=*dst;
    int found=0;

    /* Check arguments */
    if (name==NULL)
	return -1;
    namelen=strlen(name);

    /* look for existing entry */
    for (i=0; newenv[i]; i++)  {
	if (strncmp(name,newenv[i],namelen)==0 && newenv[i][namelen]=='=')   {
	    found=1;
	    entry=i;
	    break;
	}
    }
    /* no match */
    if (found==0) return 0;
    
    /* Remainder of list... */
    for (; newenv[i]; i++);

    /* Now clean entry if it's there and swap with the last in the list */
    free(newenv[entry]);

    /* If there were more than one elements, move the last to the removed one */
    if (i>0)    {
	newenv[entry]=newenv[i-1]; /* last element */
	newenv[i-1]=NULL;
    } else
	newenv[entry]=NULL;
    return 0;
}

/**
 * Removes all variables starting with pattern from dst as in
 * cgul_clear_env_pattern() with an 'external environment'.
 * Returns number of cleaned elements, -1 on general error, -2 on varname too
 * long, INT_MAX on more values than INT_MAX.
 */
int cgul_clear_env_pattern_dst(env_t *dst, const_string_t pattern)  {
    int rc;
    size_t i=0, match=0,patlen;
    env_t newenv=*dst;

    if (!pattern)
	return -1;
    patlen=strlen(pattern);
    while(newenv[i])	{
	if (strncasecmp(pattern,newenv[i],patlen)==0)  {
	    if ((rc=cgul_unsetenvpair_dst(&newenv,newenv[i])))
		return rc;
	    match++;
	} else
	    i++; /* only increase when we haven't just removed an entry... */
    }
    /* Check that match fits in a INT_MAX */
    return (match>INT_MAX ? INT_MAX : (int)match);
}

/**
 * Adds to the current environment the var=value pairs from src which have
 * varname in list. Returns number of entries added or -1 on error.
 * NOTE: we do a strdup before the actual putenv, so even though putenv doesn't
 * reserve new memory, we stil can free src !
 * Returns number of entries added, -1 on error, INT_MAX on more values than
 * INT_MAX
 */
int cgul_putenv_src_list(const_env_t src, const_env_t list)	{
    int src_idx;
    size_t i,match=0;
    char *dummy;

    /* Only do when list exists...*/
    if (!list)
	return 0;

    /* loop over list */
    for (i=0; list[i]; i++)   {
	if ((src_idx=cgul_getvarindex(src,list[i]))>=0)	{
	    /* Found match */
	    /* Make copy, so that src eventually can be freed! */
	    if ( (dummy=strdup(src[src_idx]))==NULL )
		return -1;
	    /* Now do the putenv */
	    if (putenv(dummy))
		return -1;
	    match++;
	}
    }
    /* Check that match fits in a INT_MAX */
    return (match>INT_MAX ? INT_MAX : (int)match);
}

/**
 * Adds to the current environment the var=value pairs from src which start with
 * pattern. Returns number of entries added or -1 on error.
 * NOTE: we do a strdup before the actual putenv, so even though putenv doesn't
 * reserve new memory, we stil can free src !
 * Returns number of entries added, -1 on error, INT_MAX on more values than
 * INT_MAX
 */
int cgul_putenv_src_pattern(const_env_t src, const_string_t pattern)	{
    size_t i,match=0,len;
    char *dummy;
    
    if (!pattern)
	return -1;
    len=strlen(pattern);
    /* loop over src environ */
    for (i=0; src[i]; i++)  {
	if (strncmp(src[i],pattern,len)==0) {
	    /* Found match */
	    /* Make copy, so that src eventually can be freed! */
	    if ( (dummy=strdup(src[i]))==NULL )
		return -1;
	    /* Now do the putenv */
	    if (putenv(dummy))
		return -1;
	    match++;
	}
    }
    /* Check that match fits in a INT_MAX */
    return (match>INT_MAX ? INT_MAX : (int)match);
}

