#!/bin/bash

#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors:
# 2009-
#      Oscar Koeroo <okoeroo@nikhef.nl>
#      Mischa Sall\'e <msalle@nikhef.nl>
#      NIKHEF Amsterdam, the Netherlands
#

#################
# Setup options #
#################

CONTINUEONERROR=no
#CONTINUEONERROR=yes

# Account to run under
TEST_ACCOUNT="nobody"
TEST_ID=$(id -u ${TEST_ACCOUNT} 2>/dev/null)
if [ -z "$TEST_ID" ];then
    echo "ERROR: account ${TEST_ACCOUNT} does not exist" >&2
    exit 1
fi
CLIENT_CERT="/tmp/x509up_u${TEST_ID}"

# Where installation is living
PREFIX="/usr"
LIBDIR="${PREFIX}/lib64"

# gLExec binary
GLEXEC_EXEC="${PREFIX}/sbin/glexec"

# Installation dependent settings
GLEXEC_CONF="/etc/glexec.conf"

LCAS_PLUGIN_PATH="${LIBDIR}/lcas/"
LCMAPS_PLUGIN_PATH="${LIBDIR}/lcmaps/"
CAPATH="/etc/grid-security/certificates"

# LCAS settings
USE_LCAS="no"

# Scas settings
USE_SCAS="yes"
#SCAS_ENDPOINT="https://graszaad.nikhef.nl:8443"
SCAS_ENDPOINT="https://grasmaaier.nikhef.nl:8443"

# Argus settings
USE_PEPC="yes"
PEPD_ENDPOINT="https://argus-el6-64.testbed:8154/authz"
PEPD_RESOURCE_ID="wn"
PEPD_ACTION_ID="http://glite.org/xacml/action/execute"
PEPD_RUNNING_FROM_EL6="yes"

# Target account options settings
LOCALACCOUNT_TEST_MAP_USER="pvier001"
POOLACCOUNT_TEST_MAP_USER=".pvier"

#######################################################
# The remaining settings probably don't need changing #
#######################################################

# gLExec settings in setuid/non-setuid mode
GLEXEC_OWNERSHIP_SETUID="root:root"
GLEXEC_FILE_PERM_SETUID="4711"
GLEXEC_OWNERSHIP_NON_SETUID="root:root"
GLEXEC_FILE_PERM_NON_SETUID="0711"

# glexec.conf settings in setuid/non-setuid mode
CONF_OWNERSHIP_SETUID="glexec:root"
CONF_FILE_PERM_SETUID="0400"
CONF_OWNERSHIP_NON_SETUID="glexec:root"
CONF_FILE_PERM_NON_SETUID="0444"

# testfile permissions/ownership
TESTFILE_OWNERSHIP="root:root"

# Test we are actually root
if [ `id -u` -ne 0 ]; then
    echo "ERROR: test script needs to run as root" >&2
    exit 1
fi

test_dir="/tmp/glexec_compound"
if [ -d "${test_dir}" ]; then
    read -p "Directory ${test_dir} already exists, replace (y/N)? " choice
    if [ "${choice}" = "y" -o "${choice}" = "Y" ];then
	rm -rf ${test_dir}
    else
	exit 0
    fi
fi
mkdir -p ${test_dir}/log ${test_dir}/lcas ${test_dir}/lcmaps ${test_dir}/tmp
chmod 1777 ${test_dir}/tmp

test_glexec_conf_backup="${test_dir}/glexec.conf.backup"
test_glexec_logfile="${test_dir}/log/glexec.log"
test_glexec_log_level="3"

test_lcas_log_file="${test_dir}/log/lcas_lcmaps.log"
test_lcas_db="${test_dir}/lcas/lcas-testing.db"
test_lcas_userban_file="${test_dir}/lcas/userban.db"
test_lcas_debug_level="0"

test_lcmaps_db="${test_dir}/lcmaps/lcmaps-testing.db"
test_lcmaps_log_file="${test_dir}/log/lcas_lcmaps.log"
test_lcmaps_debug_level="3"

priv_sep_file="${test_dir}/glexec_priv_sep_test.sh"

testfile="${test_dir}/glexec_test_file_to_test_with.sh"

GLEXEC_TEST_GRID_MAPFILE="${test_dir}/glexec-test-grid-mapfile"

# Different proxies
USER_PROXY="$CLIENT_CERT"
X509_USER_PROXY="$CLIENT_CERT"
SOURCE_PROXY="$CLIENT_CERT"
TARGET_PROXY="${test_dir}/tmp/target_proxy"

# Read and set the $SUBJECT_DN from the $CLIENT_CERT
SUBJECT_DN=`openssl x509 -subject -noout -in ${CLIENT_CERT} | sed -e 's/subject=\ //' -e 's/\/CN=proxy//g'`
if [ "$SUBJECT_DN" == "" ]; then
    echo No Subject DN registered
    exit 1
fi
echo "Subject DN to use: $SUBJECT_DN"

# Backup file and old mode of glexec
glexec_mode=$(stat -c %a ${GLEXEC_EXEC} 2>/dev/null)
if [ -z "${glexec_mode}" ];then
    echo "ERROR: cannot stat $GLEXEC_EXEC" >&2
    exit 1
fi
glexec_owner=$(stat -c %U:%G  ${GLEXEC_EXEC} 2>/dev/null)
/bin/cp -pf ${GLEXEC_CONF} ${test_glexec_conf_backup}

#####################################################################
#####################################################################

#Setup the LCAS ban file
touch ${test_lcas_userban_file}

restore_glexec_settings()   {
    # Put back original glexec.conf and mode/ownership of glexec
    echo "Putting back ${GLEXEC_CONF} from ${test_glexec_conf_backup}"
    /bin/cp -fp ${test_glexec_conf_backup} ${GLEXEC_CONF}
    echo "Reverting ownership of ${GLEXEC_EXEC} to ${glexec_owner}"
    chown ${glexec_owner} ${GLEXEC_EXEC}
    echo "Reverting mode of ${GLEXEC_EXEC} to ${glexec_mode}"
    chmod ${glexec_mode} ${GLEXEC_EXEC}
}

# Signal handler: set back the settings and exit
sighandler()	{
    echo "Caught signal"
    restore_glexec_settings
    # exit with 128+SIGINT
    exit 130
}

# Catch SIGINT and set back the 
trap sighandler SIGINT

create_glexec_conf_file() {
    cat > $GLEXEC_CONF <<End-of-message
#
#  Glexec
#
[glexec]
log_destination              = $LOG_DESTINATION
log_level                    = $LOG_LEVEL
log_file                     = $LOG_FILE

linger                       = $LINGER
silent_logging               = $SILENT_LOGGING
user_white_list              = $USER_WHITE_LIST

preserve_env_variables       = $PRESERVE_ENV_VARIABLES
user_identity_switch_by      = $USER_IDENTITY_SWITCH_BY

prohibit_exec_via_symlink    = $PROHIBIT_EXEC_VIA_SYMLINK
pedantic_security_checks     = $PEDANTIC_SECURITY_CHECKS

input_lock_mechanism         = $INPUT_LOCK_MECHANISM
target_lock_mechanism        = $TARGET_LOCK_MECHANISM

use_lcas                     = $USE_LCAS

lcas_libdir                  = ${LIBDIR}
lcas_db_file                 = $LCAS_DB_FILE
lcas_log_file                = $LCAS_LOG_FILE
lcas_debug_level             = $LCAS_DEBUG_LEVEL
#lcas_log_level               = $LCAS_LOG_LEVEL

lcmaps_libdir                = ${LIBDIR}
lcmaps_db_file               = $LCMAPS_DB_FILE
lcmaps_get_account_policy    = $LCMAPS_GET_ACCOUNT_POLICY
lcmaps_log_file              = $LCMAPS_LOG_FILE
lcmaps_debug_level           = $LCMAPS_DEBUG_LEVEL
#lcmaps_log_level             = $LCMAPS_LOG_LEVEL

End-of-message

    chown $1 $3
    chmod $2 $3
}


create_lcas_db_file() {
    cat > $test_lcas_db <<End-of-message
# LCAS policy file/plugin definition
# Written by: Oscar Koeroo - okoeroo * at * nikhef * dot * nl
pluginname=${LCAS_PLUGIN_PATH}lcas_userban.mod,pluginargs=${test_lcas_userban_file}
End-of-message
}

create_lcmaps_db_file() {
    # Write a dummy grid-map
    cat > ${GLEXEC_TEST_GRID_MAPFILE} <<End-of-message
"${SUBJECT_DN}" ${LOCALACCOUNT_TEST_MAP_USER}
"${SUBJECT_DN}" ${POOLACCOUNT_TEST_MAP_USER}
End-of-message

    # Write the lcmaps.db file for testing, filled with various different
    # scenarios

    ### Plugin inits ###
    cat > $test_lcmaps_db <<End-of-message
# LCMAPS policy file/plugin definition
# Written by: Oscar Koeroo - okoeroo * at * nikhef * dot * nl
# The configuration file is specialized for non-root privileged processes/services, like:
### The WMProxy
### SCAS service
### gLExec in logging only mode (when it supports multiple evaluation policies)


# default path for the modules
path = $LCMAPS_PLUGIN_PATH

# Plugin definitions:
#good             = "lcmaps_dummy_good.mod"

good             = "lcmaps_dummy_good.mod"
                   " --dummy-username nobody"
                   " --dummy-group nobody"
                   " --dummy-sec-group nobody"

posix_enf        = "lcmaps_posix_enf.mod"
                   " -maxuid 1"
                   " -maxpgid 1"
                   " -maxsgid 32"

localaccount     = "lcmaps_localaccount.mod"
                   "-gridmapfile $GLEXEC_TEST_GRID_MAPFILE"

verifyproxy = "lcmaps_verify_proxy.mod"
             " -certdir $CAPATH"
#             "--discard_private_key_absence"
#             "--never_discard_private_key_absence"

End-of-message


    if [ -n "${USE_SCAS}" ] && [ "${USE_SCAS}" = "yes" ]; then
        cat >> $test_lcmaps_db <<End-of-message
scasclient = "lcmaps_scas_client.mod"
             "-capath $CAPATH"
#             " -cert   /etc/grid-security/hostcert.pem"
#             " -key    /etc/grid-security/hostkey.pem"
             "--endpoint $SCAS_ENDPOINT"
#             "--retry 5"
#             "--endpoint-strategy round-robin"
#             "--endpoint-strategy round-robin-random-start"
#             "--endpoint-strategy random"
             "-resourcetype wn"
             "-actiontype execute-now"
End-of-message
    fi

    if [ -n "${USE_PEPC}" ] && [ "${USE_PEPC}" = "yes" ]; then
	cat >> $test_lcmaps_db <<End-of-message
pepc        = "lcmaps_c_pep.mod"
              "--pep-daemon-endpoint-url $PEPD_ENDPOINT"
              "--resourceid $PEPD_RESOURCE_ID"
              "--actionid $PEPD_ACTION_ID"
              "--capath /etc/grid-security/certificates"
              "--pep-certificate-mode implicit"
End-of-message

	if [ -n "${PEPD_RUNNING_FROM_EL6}" ] && [ "${PEPD_RUNNING_FROM_EL6}" = "yes" ]; then
	    cat >> $test_lcmaps_db <<End-of-message
              " --use-pilot-proxy-as-cafile" # Add this on RHEL 6 based systems
End-of-message
	fi
    fi

    ### Plugin Policies ###

    if [ -n "${USE_SCAS}" ] && [ "${USE_SCAS}" = "yes" ]; then
	cat >> $test_lcmaps_db <<End-of-message

scas_posix:
verifyproxy -> scasclient
scasclient -> posix_enf

scas_noposix:
verifyproxy -> scasclient
End-of-message
    fi

    if [ -n "${USE_PEPC}" ] && [ "${USE_PEPC}" = "yes" ]; then
	cat >> $test_lcmaps_db <<End-of-message

argus_posix:
verifyproxy -> pepc
pepc -> posix_enf

argus_noposix:
verifyproxy -> pepc
End-of-message
    fi

    cat >> $test_lcmaps_db <<End-of-message

localmap_posix:
verifyproxy -> localaccount
localaccount -> posix_enf

localmap_noposix:
verifyproxy -> localaccount

End-of-message

}


setup_glexec_conf() {
	LOG_DESTINATION="file"
	LOG_LEVEL="${test_glexec_log_level}"
	LOG_FILE="${test_glexec_logfile}"

	LINGER="yes"
	SILENT_LOGGING="no"
	USER_WHITE_LIST="$TEST_ACCOUNT"

	PRESERVE_ENV_VARIABLES="X509_USER_CERT, X509_USER_KEY"
	USER_IDENTITY_SWITCH_BY="lcmaps"

	PROHIBIT_EXEC_VIA_SYMLINK="no"
	PEDANTIC_SECURITY_CHECKS="no"

	INPUT_LOCK_MECHANISM="fcntl"
	TARGET_LOCK_MECHANISM="fcntl"

	#USE_LCAS

	LCAS_DB_FILE="${test_lcas_db}"
	LCAS_LOG_FILE="${test_lcas_log_file}"
#	LCAS_LOG_LEVEL="1"
	LCAS_DEBUG_LEVEL="${test_lcas_debug_level}"

	LCMAPS_DB_FILE="${test_lcmaps_db}"
	LCMAPS_LOG_FILE="${test_lcmaps_log_file}"
#	LCMAPS_LOG_LEVEL="1"
	LCMAPS_DEBUG_LEVEL="${test_lcmaps_debug_level}"
	LCMAPS_GET_ACCOUNT_POLICY="scas_posix"
}


write_cmd_to_file() {
    cat > $priv_sep_file <<End-of-message
#!/bin/sh

export X509_USER_PROXY=$X509_USER_PROXY

export GLEXEC_CLIENT_CERT=$CLIENT_CERT
export GLEXEC_SOURCE_PROXY=$SOURCE_PROXY
export GLEXEC_TARGET_PROXY=$TARGET_PROXY

cd \$HOME

echo    "                          Executing: $1"
# echo -n "                          "
$1

exit \$?

End-of-message

    chmod +x $priv_sep_file
}

#####################################################################

#The deployment scenarios:
#1.  setuid     / SCAS     / file logging / linger
#2.  setuid     / SCAS     / syslog       / linger
#3.  non-setuid / SCAS     / syslog       / linger
#4.  setuid     / non-SCAS / file logging / linger
#5.  setuid     / non-SCAS / syslog       / linger
#6.  non-setuid / non-SCAS / syslog       / linger
#7.  setuid     / SCAS     / file logging / don't linger
#8.  setuid     / SCAS     / syslog       / don't linger
#9.  non-setuid / SCAS     / syslog       / don't linger
#10. setuid     / non-SCAS / file logging / don't linger
#11. setuid     / non-SCAS / syslog       / don't linger
#12. non-setuid / non-SCAS / syslog       / don't linger
#13. setuid     / PEPC     / file logging / linger
#14. setuid     / PEPC     / syslog       / linger
#15. non-setuid / PEPC     / syslog       / linger

#16. setuid     / PEPC     / file logging / don't linger
#17. setuid     / PEPC     / syslog       / don't linger
#18. non-setuid / PEPC     / syslog       / don't linger


setup_glexec_deployment_scene_1() {
    echo "Deployment scenario  1 :: setting up: setuid, SCAS, file logging, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_2() {
    echo "Deployment scenario  2 :: setting up: setuid, SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_3() {
    echo "Deployment scenario  3 :: setting up: non-setuid, SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_4() {
    echo "Deployment scenario  4 :: setting up: setuid, non-SCAS, file logging, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_5() {
    echo "Deployment scenario  5 :: setting up: setuid, non-SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_6() {
    echo "Deployment scenario  6 :: setting up: non-setuid, non-SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_7() {
    echo "Deployment scenario  7 :: setting up: setuid, SCAS, file logging, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_8() {
    echo "Deployment scenario  8 :: setting up: setuid, SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_9() {
    echo "Deployment scenario  9 :: setting up: non-setuid, SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_10() {
    echo "Deployment scenario 10 :: setting up: setuid, non-SCAS, file logging, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_11() {
    echo "Deployment scenario 11 :: setting up: setuid, non-SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_12() {
    echo "Deployment scenario 12 :: setting up: non-setuid, non-SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_13() {
    echo "Deployment scenario 13 :: setting up: setuid, PEPC, file logging, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="argus_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_14() {
    echo "Deployment scenario 14 :: setting up: setuid, PEPC, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="argus_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_15() {
    echo "Deployment scenario 15 :: setting up: non-setuid, PEPC, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="argus_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_16() {
    echo "Deployment scenario 16 :: setting up: setuid, PEPC, file logging, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="argus_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_17() {
    echo "Deployment scenario 17 :: setting up: setuid, PEPC, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="argus_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

setup_glexec_deployment_scene_18() {
    echo "Deployment scenario 18 :: setting up: non-setuid, PEPC, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="argus_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $GLEXEC_CONF

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}


#####################################################################

test_result_expectation_handler() {
    if [ $1 -ne $2 ]; then
        echo
        echo -n "--- Test failure detected! Test returned $1, the expected result was $2 ---"
        if [ ! "$CONTINUEONERROR" == "yes" ]; then
	    echo
            echo
	    echo "For details see"
	    echo " ${test_glexec_logfile}"
	    echo " ${test_lcmaps_log_file}"
	    echo "for details"
	    echo
            restore_glexec_settings
            exit 1;
        else
            echo " continuing..."
            echo
        fi
    fi
}

execute_cmd_from_test_account() {

    EXPECTED_RESULT=$1

    # Shift input variables to seperate the commands and arguments from the $1 value
    shift

    # Write the command into a script. This script will be executed as the $TEST_ACCOUNT from $TEST_ACCOUNT's environment
    write_cmd_to_file "$*"

    echo "Executing deployment scenarios :: su - $TEST_ACCOUNT $priv_sep_file :: \"$*\""

    ### Deployment scenario 1 ###
    if [ -n "${USE_SCAS}" ] && [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_1
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  1 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 2 ###
    if [ -n "${USE_SCAS}" ] && [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_2
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  2 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 3 ###
    if [ -n "${USE_SCAS}" ] && [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_3
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  3 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 4 ###
    setup_glexec_deployment_scene_4
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  4 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 5 ###
    setup_glexec_deployment_scene_5
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  5 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 6 ###
    setup_glexec_deployment_scene_6
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  6 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 7 ###
    if [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_7
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  7 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 8 ###
    if [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_8
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  8 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 9 ###
    if [ "${USE_SCAS}" = "yes" ]; then
        setup_glexec_deployment_scene_9
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario  9 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 10 ###
    setup_glexec_deployment_scene_10
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario 10 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 11 ###
    setup_glexec_deployment_scene_11
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario 11 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 12 ###
    setup_glexec_deployment_scene_12
    rm -f $TARGET_PROXY

    su - $TEST_ACCOUNT $priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario 12 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 13 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_13
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 13 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 14 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_14
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 14 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 15 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_15
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 15 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 16 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_16
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 16 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 17 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_17
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 17 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi


    ### Deployment scenario 18 ###
    if [ "${USE_PEPC}" = "yes" ]; then
        setup_glexec_deployment_scene_18
        rm -f $TARGET_PROXY

        su - $TEST_ACCOUNT $priv_sep_file
        rc=$?

        # Grab the result and compare with the expected result
        echo "Deployment scenario 18 :: Result: $rc - Expected: $EXPECTED_RESULT"
        test_result_expectation_handler $rc $EXPECTED_RESULT
    fi



    return $rc
}

#####################################################################

test_1() {
    echo "Test 1: Basic functional gLExec test (all must be working)"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /usr/bin/id -a
}

test_2() {
    echo "Test 2: gLExec executing a non-existing executable"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC I_do_not_exist
}

test_3() {
    echo "Test 3: gLExec try to execute a symlinked binary (success)"

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PROHIBIT_EXEC_VIA_SYMLINK="no"

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /bin/sh -c echo hello glexec world
}

test_4() {
    echo "Test 4: gLExec tries to execute a symlinked binary (prohibited/protected)"

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PROHIBIT_EXEC_VIA_SYMLINK="yes"

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC /bin/sh -c echo hello glexec world
}

test_5_1() {
    echo "Test 5_1: gLExec tries to execute a file that isn't executable"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    > $TEST_FILE1
    chmod a-x $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_5_2() {
    echo "Test 5_2: gLExec tries to execute a file that isn't a file"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    mkfifo $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_6() {
    echo "Test 6: gLExec tries to execute a file that isn't executable, but now the pedantic checks are enabled, still same return value"

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="${testfile}"
    > $TEST_FILE1
    chmod a-x $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_7() {
    echo "Test 7: gLExec tries to execute a file that is executable, but now unreachable by ownership and file permissions."

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="no"

    TEST_FILE1="${testfile}"
    > $TEST_FILE1
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 0750 $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_8() {
    echo "Test 8: gLExec tries to execute a file that is executable, but now unreachable by ownership and file permissions, with pedenatic file checks on"

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="${testfile}"
    > $TEST_FILE1
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 0750 $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_9() {
    echo "Test 9: gLExec testing lock mechansims: input locking: $1 for target locking: $2"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    INPUT_LOCK_MECHANISM=$1
    TARGET_LOCK_MECHANISM=$2

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /usr/bin/id -a
}

test_10() {
    echo "Test 10: gLExec tries to execute a setuid-enabled executable"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 4555 $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_11() {
    echo "Test 11: gLExec tries to execute a setgid-enabled executable."

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    rm $TEST_FILE1
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 2555 $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_12() {
    echo "Test 12: gLExec tries to execute a setuid-setgid-enabled executable."

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    rm $TEST_FILE1
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 6555 $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_13() {
    echo "Test 13: gLExec tries to execute a world-writeable executable."

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    TEST_FILE1="${testfile}"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 0777  $TEST_FILE1

    # execute test in deployment scenarios
    # execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1
    execute_cmd_from_test_account 0 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_14() {
    echo "Test 14: gLExec tries to execute a world-writeable executable, with pedantic checks are now enabled."

    # Restore settings to default before compound test is executed
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="${testfile}"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chown $TESTFILE_OWNERSHIP $TEST_FILE1
    chmod 0777  $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

test_15() {
    echo "Test 1: Regression test for https://bugzilla.nikhef.nl/show_bug.cgi?id=16 (all must be working)"

    # Restore settings to default before compound test is executed
    setup_glexec_conf

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   MALLOC_TEST=bla $GLEXEC_EXEC /usr/bin/id -a
}



test_execution_of_all_tests_in_all_scenarios() {
    echo "Starting all tests..."

    echo "----- test_1 -----"
    test_1

    echo "----- test_2 -----"
    test_2

    echo "----- test_3 -----"
    test_3

    echo "----- test_4 -----"
    test_4

    echo "----- test_5_1 -----"
    test_5_1

    echo "----- test_5_2 -----"
    test_5_2

    echo "----- test_6 -----"
    test_6

    echo "----- test_7 -----"
    test_7

    echo "----- test_8 -----"
    test_8

    echo "----- test_9.1 -----"
    test_9 disabled disabled

    echo "----- test_9.2 -----"
    test_9 flock    disabled

    echo "----- test_9.3 -----"
    test_9 disabled flock

    echo "----- test_9.4 -----"
    test_9 flock    flock

    echo "----- test_9.5 -----"
    test_9 fcntl    disabled

    echo "----- test_9.6 -----"
    test_9 disabled fcntl

    echo "----- test_9.7 -----"
    test_9 fcntl    flock

    echo "----- test_9.8 -----"
    test_9 flock    fcntl

    echo "----- test_9.9 -----"
    test_9 fcntl    fcntl

    echo "----- test_10 -----"
    test_10

    echo "----- test_11 -----"
    test_11

    echo "----- test_12 -----"
    test_12

    echo "----- test_13 -----"
    test_13

    echo "----- test_14 -----"
    test_14
    
    echo "----- test_15 -----"
    test_15
}


#####################################################################
# Start of the program
#####################################################################

echo "This is the gLExec testing facility"

echo "Starting test sequence..."

echo "Showing compile options and version:"
$GLEXEC_EXEC -v
$GLEXEC_EXEC -V

echo "Setup gLExec configuration file basic values"
setup_glexec_conf


echo "Setting up configuration for LCAS to '${test_lcas_db}'"
create_lcas_db_file
echo "Setting up configuration for LCMAPS to '${test_lcmaps_db}'"
create_lcmaps_db_file


#####################################################################
# Execute all tests
#####################################################################
test_execution_of_all_tests_in_all_scenarios


#####################################################################
# Finish
#####################################################################
echo
echo
if [ "$CONTINUEONERROR" == "yes" ]; then
    echo "Test sequence finished with UNKNOWN success, because \$CONTINUEONERROR=\"yes\""
else
    echo "Test sequence finished with SUCCESS."
fi

restore_glexec_settings

